var createError = require("http-errors");
var express = require("express");
var path = require("path");
var cookieParser = require("cookie-parser");
var browserify = require("browserify-middleware");
var lessMiddleware = require("less-middleware");
var logger = require("morgan");

var indexRouter = require("./routes/index");
var keyRouter = require("./routes/key");
var paypalCheckoutRouter = require("./routes/checkout/paypal.js");
//var checkoutRouter = require("./routes/checkout/checkout");
var redeemRouter = require("./routes/redeem.js");

var app = express();

// view engine setup
app.set("views", path.join(__dirname, "views"));
app.set("view engine", "ejs");

app.use(logger("dev"));
app.use(express.json());
app.use(express.urlencoded({ extended: false }));
app.use(cookieParser());
app.use(lessMiddleware(path.join(__dirname, "public")));
app.use(express.static(path.join(__dirname, "public")));

app.use("/", indexRouter);
app.use("/key", keyRouter);
//app.use("/checkout", checkoutRouter);
app.use("/redeem", redeemRouter);

// Browserified Javascript files
app.get(
  "/js/base.js",
  browserify(path.join(__dirname, "resources", "js", "base.js"))
);

app.get(
  "/js/funding_sources.js",
  browserify(path.join(__dirname, "resources", "js", "funding_sources.js"))
);

app.get(
  "/js/checkout_paypal.js",
  browserify(path.join(__dirname, "resources", "js", "checkout_paypal.js"))
);

app.use("/webhooks/paypal", paypalCheckoutRouter);
app.use("/js/paypal", paypalCheckoutRouter);
// catch 404 and forward to error handler
app.use(function (req, res, next) {
  next(createError(404));
});

// error handler
app.use(function (err, req, res, next) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = req.app.get("env") === "development" ? err : {};

  // render the error page
  res.status(err.status || 500);
  res.render("error");
});

module.exports = app;
