class Key {
  static get DATABASE_PREFIX() {
    return "key:";
  }

  static get REDIS_CLIENT() {
    let config = require("config");
    let Redis = require("ioredis");

    return new Redis({
      host: config.get("redis.host"),
    });
  }

  static get EXPIRATION_AFTER_CHARGE_DAYS() {
    return 365;
  }

  static get KEY_CHARSET() {
    return require("config").get("keys.charset");
  }

  static get KEY_LENGTH() {
    return require("config").get("keys.length");
  }

  static async EXISTS(key) {
    return new Promise((resolve, reject) => {
      let redis_client = Key.REDIS_CLIENT;
      redis_client.exists(Key.DATABASE_PREFIX + key).then((result) => {
        console.log(result);
        resolve(result);
      });
    });
  }

  /**
   *
   * @returns A new MetaGer-Pass Key with 0 searches
   */
  static async GET_NEW_KEY() {
    return new Promise(async (resolve, reject) => {
      let redis_client = Key.REDIS_CLIENT;

      do {
        let crypto = require("crypto");
        let key = crypto.randomUUID();

        let key_exists = await redis_client.exists(Key.DATABASE_PREFIX + key);
        if (!key_exists) {
          return resolve(key);
        }
        await new Promise((resolve) => setTimeout(resolve, 50));
      } while (true);
    });
  }

  static async GET_KEY_CHARGE(key) {
    let redis_client = Key.REDIS_CLIENT;
    return redis_client.get(Key.DATABASE_PREFIX + key).then((key_charge) => {
      return key_charge | 0;
    });
  }

  static async CHARGE_KEY(key, amount) {
    let redis_client = Key.REDIS_CLIENT;
    let expiration = require("dayjs")().add(
      Key.EXPIRATION_AFTER_CHARGE_DAYS,
      "day"
    );
    // Check if key exists and is eligable for recharge
    return redis_client
      .pipeline()
      .incrby(Key.DATABASE_PREFIX + key, amount)
      .expireat(Key.DATABASE_PREFIX + key, expiration.unix())
      .exec();
  }

  static async DISCHARGE_KEY(key, amount) {
    let redis_client = Key.REDIS_CLIENT;

    // Check if key exists and get charge
    redis_client.get(Key.DATABASE_PREFIX + key).then((current_amount) => {
      if (current_amount && current_amount > 0) {
        if (current_amount > amount) {
          let expiration = require("dayjs")().add(
            Key.EXPIRATION_AFTER_CHARGE_DAYS,
            "day"
          );
          return redis_client
            .pipeline()
            .set(Key.DATABASE_PREFIX + key, current_amount - amount)
            .expireat(Key.DATABASE_PREFIX + key, expiration.unix())
            .exec();
        } else {
          return redis_client.del(Key.DATABASE_PREFIX + key);
        }
      } else {
        throw "Key Does not exist or is not charged";
      }
    });
  }
}

module.exports = Key;
