var express = require("express");
var router = express.Router();
const { param, validationResult } = require("express-validator");
const config = require("config");

var orderRouter = require("./orders");
var checkout_router_paypal = require("./checkout/paypal");

var Key = require("../app/Key");

router.get("/create", function (req, res, next) {
  Key.GET_NEW_KEY().then((key) => {
    res.redirect("/key/" + key + "?new=true");
  });
});

router.use("/:key", param("key").isUUID(4), async (req, res, next) => {
  // Input Validation
  const errors = validationResult(req);
  if (!errors.isEmpty()) {
    return res.status(400).json({ errors: errors.array() });
  }

  let metager_url =
    "https://metager.de/meta/settings/load-settings?key=" +
    encodeURIComponent(req.params.key);
  let QRCode = require("qrcode");

  let qr_data_uri = await QRCode.toDataURL(metager_url);

  req.data = {
    created_new: req.query.new === "true" ? true : false,
    key: {
      key: req.params.key,
      charge: await Key.GET_KEY_CHARGE(req.params.key),
      settings_url: metager_url,
      qr: qr_data_uri,
    },
    js: [],
    css: ["/styles/key/key.css"],
  };
  next("route");
});

// Basic account page
router.get("/:key", async (req, res) => {
  res.render("key", req.data);
});

router.use("/:key/orders", orderRouter);

/**
 * Validate Amount field for checkout process
 */
router.use(
  "/:key/checkout/:amount?",
  param("amount")
    .optional({ checkFalsy: true })
    .isInt()
    .toInt()
    .isIn([300, 600, 900, 1200, 1800, 3600]),
  (req, res, next) => {
    // Input Validation
    const errors = validationResult(req);
    if (!errors.isEmpty()) {
      return res.status(400).json({ errors: errors.array() });
    }
    let amount = req.params.amount;
    if (typeof amount !== "undefined" && parseInt(req.params.amount) === 0) {
      amount = undefined;
    } else {
      amount = parseInt(amount);
    }
    req.data.checkout = {
      amount: amount,
    };

    // Add a URL to change the checkout amount
    req.data.change_url = {
      amount: "/key/" + encodeURIComponent(req.data.key.key) + "#charge",
    };

    next("route");
  }
);

router.get("/:key/checkout/:amount", (req, res) => {
  if (req.query.error) {
    req.data.checkout.error = req.query.error;
  }
  req.data.js.push("/js/funding_sources.js");
  res.render("key", req.data);
});

router.use("/:key/checkout/:amount?/paypal", checkout_router_paypal);

module.exports = router;
